#ifndef KENNY_DEFS_H

#define KENNY_DEFS_H


#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#include <conio.h>
#undef CDECL
#define CDECL __cdecl
typedef unsigned long long U64;
typedef unsigned long long BitMap;
typedef short SHORTINT;
typedef unsigned short USHORTINT;
typedef int BOOLTYPE;

#else

#include <stdint.h>
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <cstring>
#include <sys/select.h>
typedef uint64_t U64;
typedef uint64_t BitMap;
typedef short SHORTINT;
typedef unsigned short USHORTINT;
typedef int BOOLTYPE;
#undef CDECL
#define CDECL
#define _fileno fileno
#define _dup dup
#define _dup2 dup2
#define _close close


static int _kbhit(void)
{
  struct termios oldt, newt;
  int ch;
  int oldf;

  tcgetattr(STDIN_FILENO, &oldt);
  newt = oldt;
  newt.c_lflag &= ~(ICANON | ECHO);
  tcsetattr(STDIN_FILENO, TCSANOW, &newt);
  oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
  fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

  ch = getchar();

  tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
  fcntl(STDIN_FILENO, F_SETFL, oldf);

  if(ch != EOF)
  {
    ungetc(ch, stdin);
    return 1;
  }

  return 0;
}

/******* Common definitions and typedefs ***********************************/

  typedef int             BOOL;



  /****** Simple typedefs  ***************************************************/

  typedef unsigned char   BYTE;           //  8-bit
  typedef unsigned short  WORD;           // 16-bit
  typedef unsigned long   DWORD;          // 32-bit

  // QWORD is for those situation that we want
  // an 8-byte-aligned 8 byte long structure
  // which is NOT really a floating point number.
  typedef union tagUQuadWord {
      struct {
          DWORD   dwLowDword;
          DWORD   dwHighDword;
      } u;
      double      DoNotUseThisField;
  } UQuadWord;
  typedef UQuadWord       QWORD;          // 64-bit

  /****** Common pointer types ***********************************************/

  typedef unsigned long   ULONG_PTR;      // 32-bit
  typedef unsigned long   DWORD_PTR;      // 32-bit

  // boolean pointer

  typedef BYTE *           PBYTE;

  typedef WORD *           PWORD;

  typedef DWORD *          PDWORD;

  typedef QWORD *          PQWORD;

#endif

extern BOOLTYPE XB_MODE;


#ifndef _WIN32
/* Non-windows version */

static int Bioskey(void) {
  fd_set readfds;
  struct timeval timeout;
   
  FD_ZERO(&readfds);
  FD_SET(fileno(stdin), &readfds);
   /* Set to timeout immediately */
  timeout.tv_sec = 0;
  timeout.tv_usec = 0;
  select(16, &readfds, 0, 0, &timeout);
   
  return (FD_ISSET(fileno(stdin), &readfds));
}

#else
/* Windows-version */
static int Bioskey(void) {
  static int init = 0, pipe;
  static HANDLE inh;
  DWORD dw;
   /* If we're running under XBoard then we can't use _kbhit() as the input commands
    * are sent to us directly over the internal pipe */
  if (XB_MODE) {
#if defined(FILE_CNT)
    if (stdin->_cnt > 0) return stdin->_cnt;
#endif
    if (!init) {
      init = 1;
      inh = GetStdHandle(STD_INPUT_HANDLE);
      pipe = !GetConsoleMode(inh, &dw);
      if (!pipe) {
        SetConsoleMode(inh, dw & ~(ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT));
        FlushConsoleInputBuffer(inh);
      }
    }
    if (pipe) {
      if (!PeekNamedPipe(inh, NULL, 0, NULL, &dw, NULL)) return 1;
      return dw;
    }
    else {
      GetNumberOfConsoleInputEvents(inh, &dw);
      return dw <= 1 ? 0 : dw;
    }
  }
  else return _kbhit();
}
#endif













//Definitions
#define KENNY_PROG_VERSION "Kenny v0.1.1 JA, by Kenshin Himura, Copyright 2012" //String Definition
//#define KENNY_DEBUG_PERFT
//#define KENNY_DEBUG_MOVES
//#define KENNY_VERBOSE_EVAL
//#define KENNY_VERBOSE_SEE
//#define KENNY_DEBUG_EVAL
//#define KENNY_DEBUG_WINBOARD
#define KENNY_CUSTOM_VALUES
//#define KENNY_CUSTOM_POSVALS
//#define KENNY_CUSTOM_PSTABLES
//#define KENNY_CUSTOM_ENDGAME

#define MAX_CMD_BUFF     256   // Console command input buffer
#define MAX_MOV_BUFF    4096   // Max number of moves that we can store (all plies)
#define MAX_PLY           64   // Max search depth
#define MAX_GAME_LINE   1024   // Max number of moves in the (game + search) line that we can store


#endif